# 
# this file will act as a central perl type of location info database
# info is collected from:
# www.luchtkwaliteit.limburg.nl location nr < 100
# www.lml.rivm.nl website (only regional stations nr >= 100

%locations = (
    # bron: luchtkwaliteit.limburg.nl, binary search for first and last dates
    '01' => {
	name => "Buggenum", organisation => 'PLIM', id => 'NLBUG01',
	table => 'Bug',
	first => '01-04-2004',
	last => '25-01-2014',
	sense => { so2 => 0, no => 0, no2 => 0, wr => 0, ws => 0, },
	# norm => '2011',	# norm refers to last year correction factors
	},
    '02' => {
	name => "Geleen Vouershof", organisation => 'PLIM', id => 'NL50002',
	table => 'GV',
	first => '27-08-2001',
	sense => { so2 => 0, no => 0, no2 => 0, pm_10 => 0, wr => 0, ws => 0, temp => 0, },
	# norm => '2011',	# norm refers to last year correction factors
	},
    '03' => {
	name => "Geleen Asterstraat", organisation => 'PLIM', id => 'NL50003',
	table => 'GA',
	first => '01-05-2002',
	sense => { so2 => 0, no => 0, no2 => 0, pm_10 => 0, pm_25 => 0, luchtdruk => 0, },
	# norm => '2011',	# norm refers to last year correction factors
	geolocation => "50.984124N 5.823603E",
	},
    '04' => {
	name => "Maastricht A2-Nassaulaan", organisation => 'PLIM', id => 'NL50004',
	table => 'MA2',
	first => '01-05-2002',
	sense => { no => 0, no2 => 0, o3 => 0, pm_10 => 0, pm_25 => 0, zwarte_rook => 0, },
	# norm => '2011',	# norm refers to last year correction factors
	geolocation => "50.847746N 5.714768E",
	},
    '05' => {
	name => "Roermond", organisation => 'PLIM', id => 'NLROE05',
	table => 'Roer',
	first => '02-02-2008',
	last => "08-09-2010",
	sense => { no => 0, no2 => 0, temp => 0, rv => 0, },
	# norm => '2011',	# norm refers to last year correction factors
	geolocation => "51.1192N 6.0424E",
	},
    '06' => {
	name => "Horst aan de Maas", organisation => 'PLIM', id => 'NL50006',
	table => 'HadM',
	first => "3-11-2011",
	sense => { pm_10 => 0, pm_25 => 0, temp => 0, wr => 0, ws => 0, luchtdruk => 0, },
	norm => '2014',		# norm refers to last year correction factors
	2011 => { pm_10 => 0.90, pm_25 => 0.96, }, # ref prov Limburg 2013/60843
	2012 => { pm_10 => 0.92, pm_25 => 0.96, }, # ref prov Limburg 2013/60843
	2013 => { pm_10 => 0.92, pm_25 => 0.96, }, # ref prov Limburg 2014/24091
	2014 => { pm_10 => 0.92, pm_25 => 0.96, }, # ref prov Limburg 2015/P2015-0100
	geolocation => "51.455119N 6.114165E",
	},
    '07' => {
	name => "Maastricht Hoge Fronten", organisation => 'PLIM', id => 'NL50007',
	table => 'MHF',
	first => '18-04-2013',
	#last => '3-07-2014',
	sense => { pm_10 => 0, pm_25 => 0, o3 => 0, zwarte_rook => 0, temp => 0, luchtdruk => 0, },
	# norm => '2011',	# norm refers to last year correction factors
	geolocation => "50.852537N 5.680878E",
	},
    '09' => {
	name => "Maastricht Kasteel Hillenraadweg", organisation => 'PLIM', id => 'NL50009',
	table => 'MKH',
	first => '01-08-2015',
	#last => '25-11-2015',
	sense => { pm_24 => 0, zwarte_rook => 0, },
	# norm => '2011',       # norm refers to last year correction factors
	geolocation => "50.859927N 5.714018E",
	},

    # locations from www.lml.rivm.nl/table status dd 2014/07/22
    # www.lml.rivm.nl/tabel/?data=YYYYMMDDHH.tabel
    # will provide hourly unvalidated measurement values
    # but one day measurement download will cause 24 website accesses
    # for dates older as 2 months there are
    # monthly validated data: less traffic but 2 months latency
    # for monthly validated data up to 2012 there is one xls file
    # location numbers for RIVM stations are >= 100
    '107' => {
        name => 'Posterholt-Vlodropperweg', organisation => 'RIVM', id => 'NL00107',
        table => 'NL10107',
	geolocation => "51.1192N 6.0424E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { no2 => 0, no => 0, o3 => 0, },
    },
    '131' => {
        name => 'Vredepeel-Vredeweg', organisation => 'RIVM', id => 'NL00131',
        table => 'NL10131',
	geolocation => "51.5405N 5.85307E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, pm_25 => 0, no2 => 0, no => 0, o3 => 0, nh3 => 0, },
    },
    '133' => {
        name => 'Wijnandsrade-Opfergeltstraat', organisation => 'RIVM', id => 'NL00133',
        table => 'NL10133',
	geolocation => "50.9023N 5.88175E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, so2 => 0, },
    },
    '230' => {
        name => 'Biest Houtakker-Biestsestraat', organisation => 'RIVM', id => 'NL00230',
        table => 'NL10230',
	geolocation => "51.5184N 5.14845E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '235' => {
        name => 'Huijbergen-Vennekenstraat', organisation => 'RIVM', id => 'NL00235',
        table => 'NL10235',
	geolocation => "51.4344N 4.35984E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '243' => {
        name => 'De Rips-Blaarpeelweg', organisation => 'RIVM', id => 'NL00243',
        table => 'NL10243',
	geolocation => "51.5475N 5.8382E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, nh3 => 0, },
    },
    '244' => {
        name => 'De Rips-Klotterpeellaan', organisation => 'RIVM', id => 'NL00244',
        table => 'NL10244',
	geolocation => "51.5359N 5.80859E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, nh3 => 0, },
    },
    '246' => {
        name => 'Fijnaart-Zwingelspaansedijk', organisation => 'RIVM', id => 'NL00246',
        table => 'NL10246',
	geolocation => "51.6537N 4.51527E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, },
    },
    '301' => {
        name => 'Zierikzee-Lange Slikweg', organisation => 'RIVM', id => 'NL00301',
        table => 'NL10301',
	geolocation => "51.6347N 3.91662E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { no2 => 0, no => 0, o3 => 0, },
    },
    '318' => {
        name => 'Philippine-Stelleweg', organisation => 'RIVM', id => 'NL00318',
        table => 'NL10318',
	geolocation => "51.2945N 3.74948E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, so2 => 0, },
    },
    '437' => {
        name => 'Westmaas-Groeneweg', organisation => 'RIVM', id => 'NL00437',
        table => 'NL10437',
	geolocation => "51.7866N 4.45053E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '444' => {
        name => 'De Zilk-Vogelaarsdreef', organisation => 'RIVM', id => 'NL00444',
        table => 'NL10444',
	geolocation => "52.2966N 4.51082E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, nh3 => 0, so2 => 0, },
    },
    '538' => {
        name => 'Wieringerwerf-Medemblikkerweg', organisation => 'RIVM', id => 'NL00538',
        table => 'NL10538',
	geolocation => "52.8037N 5.05051E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, nh3 => 0, },
    },
    '631' => {
        name => 'Biddinghuizen-Hoekwantweg', organisation => 'RIVM', id => 'NL00631',
        table => 'NL10631',
	geolocation => "52.4487N 5.61921E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '633' => {
        name => 'Zegveld-Oude Meije', organisation => 'RIVM', id => 'NL00633',
        table => 'NL10633',
	geolocation => "52.138N 4.83819E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, nh3 => 0, co => 0, },
    },
    '644' => {
        name => 'Cabauw-Wielsekade', organisation => 'RIVM', id => 'NL00644',
        table => 'NL10644',
	geolocation => "51.9745N 4.9233E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, co => 0, so2 => 0, },
    },
    '722' => {
        name => 'Eibergen-Lintveldseweg', organisation => 'RIVM', id => 'NL00722',
        table => 'NL10722',
	geolocation => "52.0918N 6.60537E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, so2 => 0, },
    },
    '738' => {
        name => 'Wekerom-Riemterdijk', organisation => 'RIVM', id => 'NL00738',
        table => 'NL10738',
	geolocation => "52.1116N 5.70842E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, nh3 => 0, },
    },
    '807' => {
        name => 'Hellendoorn-Luttenbergerweg', organisation => 'RIVM', id => 'NL00807',
        table => 'NL10807',
	geolocation => "52.3883N 6.40293E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '818' => {
        name => 'Barsbeek-De Veenen', organisation => 'RIVM', id => 'NL00818',
        table => 'NL10818',
	geolocation => "52.6541N 6.01757E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '918' => {
        name => 'Balk-Trophornsterweg', organisation => 'RIVM', id => 'NL00918',
        table => 'NL10918',
	geolocation => "52.9169N 5.57349E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '929' => {
        name => 'Valthermond-Noorderdiep', organisation => 'RIVM', id => 'NL00929',
        table => 'NL10929',
	geolocation => "52.8757N 6.93243E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, nh3 => 0, },
    },
    '934' => {
        name => 'Kollumerwaard-Hooge Zuidwal', organisation => 'RIVM', id => 'NL00934',
        table => 'NL10934',
	geolocation => "53.3304N 6.27681E",
        norm => '2000', 2000 => { pm_10 => 1.0, pm_25 => 1.0, },
        sens => { pm_10 => 0, no2 => 0, no => 0, o3 => 0, },
    },
    '066' => {
	name => 'Nettetal-Kaldenkirchen', organisation => 'NRWF', id => 'DENW066',
	table => 'NETT',
	first => '23-08-2013',
        sense => { so2 => 0, pm_10 => 0, no => 0, no2 => 0, wr => 0, ws => 0, },
	geolocation => "51.333554N 6.197587E",
    },
);
